/* THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT! */
/* Copyright (c) 1998 SSH Communications Security Ltd. */

/* This file was generated by the SSH distribution system, and should
   not normally be edited.  However, editing this file by a customer may
   sometimes make sense to disable some features.

   This file should never be edited by anyone inside SSH
   except for testing purposes; distribution configurations should
   be defined in the distributions file. */

#ifndef SSHDISTDEFS_H
#define SSHDISTDEFS_H

#define SSHDIST_UTIL_PACKETSTREAM
#define SSHDIST_CRYPT_DL
#define SSHDIST_CRYPT_DLKEX
#define SSHDIST_CRYPT_RIPEMD160
#define SSHDIST_CRYPT_TWOFISH
#define SSHDIST_CRYPT_GENMAC
#define SSHDIST_MEDIA_ETHERNET
#define SSHDIST_UTIL_COMMON
#define SSHDIST_PGP
#define SSHDIST_CRYPT_DES
#define SSHDIST_CRYPT_GENHASH
#define SSHDIST_CRYPT_MD5
#define SSHDIST_IPSEC_FILTER_OPTIMIZER
#define SSHDIST_CRYPT_GENPKCS
#define SSHDIST_CRYPT_COMPRESS
#define SSHDIST_CRYPT_SHA
#define SSHDIST_CRYPT_BLOWFISH
#define SSHDIST_CRYPT_SSHMACS
#define SSHDIST_CRYPT_ARCFOUR
#define SSHDIST_CRYPT_DSA
#define SSHDIST_CRYPT_HMAC
#define SSHDIST_CRYPT_GENCIPH
#define SSHDIST_CRYPT_DH

#endif /* SSHDISTDEFS_H */
